import { Headers, Http } from '@angular/http';
import 'rxjs/add/operator/toPromise';

import { PagedQueryResult, PagedQueryParam } from './models';
import { Utils } from './utils';

declare var isRunning: number; //用于记录有多少ajax请求正在运行
declare var lastActiveTime: Date; // 用于记录最后活动时间
export abstract class BaseService {
	protected headers = new Headers({ 'Content-Type': 'application/json' });

	constructor(private http: Http) {
		lastActiveTime = new Date();
	}
	/**
	 * 分页查询
	 * 参数：url URL地址, queryParam：用于查询的参数。waitLoading: 是否要显示等待加载
	 * */
	protected searchPage(url: string, queryParam: PagedQueryParam, waitLoading: boolean = true): Promise<PagedQueryResult<any>> {
		if (waitLoading) {
			isRunning++;
		}

		if (!queryParam.pageNumber) {
			queryParam.pageNumber = 1;
		}
		if (!queryParam.pageSize) {
			queryParam.pageSize = 10;
		}
		if (url.indexOf("?") < 0) {
			url += "?x=1";
		}
		url += this.toUrl(queryParam, "");
		return this.http.get(url)
			.toPromise()
			.then(response => {
				if (waitLoading) {
					isRunning--;
				}
				lastActiveTime = new Date();
				let rst: PagedQueryResult<any> = response.json();
				this.processDate(rst.datas);
				return rst;
			})
			.catch(err => this.handleError(err, waitLoading));
	}
	private toUrl(obj: any, propPrefix: string): string {
		let url = "";
		let preParam = propPrefix;
		if (preParam) {
			preParam += ".";
		}
		if (obj == undefined) {
			return url;
		}
		if (typeof obj == 'object') {
			if (obj instanceof Array) {
				for (let i = 0; i < obj.length; i++) {
					if (typeof (obj[i]) == 'object') {
						url = url + this.toUrl(obj[i], propPrefix + "[" + i + "]");
					} else {
						url = url + "&" + propPrefix + "[" + i + "]" + "=" + this.formatValue(obj[i]);
					}
				}
			} else {
				for (let prop in obj) {
					if (prop != "constructor") {
						let pv = obj[prop];
						if (pv) {
							if (typeof pv == 'function') continue;
							if (typeof pv == 'object') {
								url = url + this.toUrl(pv, preParam + prop);
							} else {
								url = url + "&" + preParam + prop + "=" + this.formatValue(obj[prop]);
							}
						}
					}
				}
			}
		} else {
			url = url + "&" + propPrefix + "=" + this.formatValue(obj);
		}
		return url;
	}
	private formatValue(val: any) {
		if (val instanceof Date) {
			return val.getTime();
		}
		return encodeURI(val);
	}
    /**
     * GET请求
     * 参数：url URL地址, data：用于查询的参数。waitLoading: 是否要显示等待加载
     * */
	protected doGet(url: string, data?: any, waitLoading: boolean = true): Promise<any> {
		if (data) {
			for (let prop in data) {
				if (prop != null && prop != undefined) {
					if (url.indexOf("?") > 0) {
						url = url + "&";
					} else {
						url = url + "?";
					}
					url = url + prop + "=" + encodeURI(data[prop]);
				}
			}
		}
		if (waitLoading) {
			isRunning++;
		}
		return this.http.get(url)
			.toPromise()
			.then(response => {
				if (waitLoading) {
					isRunning--;
				}
				lastActiveTime = new Date();
				let rst = response.json();
				this.processDate(rst);
				return rst;
			})
			.catch(err => this.handleError(err, waitLoading));
	}

    /**
     * PUT请求
     * 参数：url URL地址, data：参数。waitLoading: 是否要显示等待加载
     * */
	protected doPut(url: string, data: any, waitLoading: boolean = true): Promise<any> {
		if (waitLoading) {
			isRunning++;
		}

		return this.http
			.put(url, data, { headers: this.headers })
			.toPromise()
			.then(response => {
				if (waitLoading) {
					isRunning--;
				}
				lastActiveTime = new Date();
				let rst = response.json();
				this.processDate(rst);
				return rst;
			})
			.catch(err => this.handleError(err, waitLoading));
	}
    /**
     * POST请求
     * 参数：url URL地址, data：参数。waitLoading: 是否要显示等待加载
     * */
	protected doPost(url: string, data: any, waitLoading: boolean = true): Promise<any> {
		if (waitLoading) {
			isRunning++;
		}

		return this.http
			.post(url, data, { headers: this.headers })
			.toPromise()
			.then(response => {
				if (waitLoading) {
					isRunning--;
				}
				lastActiveTime = new Date();
				let rst = response.json();
				this.processDate(rst);
				return rst;
			})
			.catch(err => this.handleError(err, waitLoading));
	}
    /**
     * DELETE请求
     * 参数：url URL地址, data：参数。waitLoading: 是否要显示等待加载
     * */
	protected doDelete(url: string, waitLoading: boolean = true): Promise<any> {
		if (waitLoading) {
			isRunning++;
		}

		return this.http.delete(url, { headers: this.headers })
			.toPromise()
			.then(response => {
				if (waitLoading) {
					isRunning--;
				}
				lastActiveTime = new Date();
				let rst = response.json();
				this.processDate(rst);
				return rst;
			})
			.catch(err => this.handleError(err, waitLoading));
	}
	protected processDate(rst: any) {
		if (rst) {
			if (Array.isArray(rst)) {
				for (let r of rst) {
					this.processDate(r);
				}
			} else {
				for (let p in rst) {
					if (typeof (rst[p]) == "object") {
						this.processDate(rst[p]);
					} else if (p.endsWith("Date") || p.endsWith("Time") || p.endsWith("Timestamp")) {
						if (typeof (rst[p]) == "number") {
							rst[p] = new Date(rst[p]);
						} else if (rst[p].length == 28 && rst[p].indexOf("T") == 10) {//"2017-11-29T12:18:54.921+0000"
							rst[p] = new Date(rst[p]);
						}

					}
				}
			}
		}
	}
	protected handleError(error: any, waitLoading: boolean): Promise<any> {
		//console.error('An error occurred:'+ error);
		if (waitLoading) {
			isRunning--;
		}
		lastActiveTime = new Date();
		let msg = Utils.resolveJsonErrorMessage(error);
		return Promise.reject(msg);
	}
}